<?php

namespace Drupal\search_api_synonym\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\search_api_synonym\Export\ExportPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The settings form for Synonym.
 *
 * @package Drupal\search_api_synonym\Form
 *
 * @ingroup search_api_synonym
 */
class SynonymSettingsForm extends ConfigFormBase {

  /**
   * An array containing available export plugins.
   *
   * @var array
   */
  protected $availablePlugins = [];

  /**
   * Logger factory service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Constructs a VacancySourceForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typed_config_manager
   *   The typed config manager.
   * @param \Drupal\search_api_synonym\Export\ExportPluginManager $manager
   *   The synonyms export plugin manager.
   * @param \Drupal\Core\File\FileSystemInterface $logger_factory
   *   The logger factory service.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, TypedConfigManagerInterface $typed_config_manager, ExportPluginManager $manager, LoggerChannelFactoryInterface $logger_factory, FileSystemInterface $file_system) {
    parent::__construct($config_factory, $typed_config_manager);

    foreach ($manager->getAvailableExportPlugins() as $id => $definition) {
      $this->availablePlugins[$id] = $manager->createInstance($id);
    }
    $this->loggerFactory = $logger_factory;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('plugin.manager.search_api_synonym.export'),
      $container->get('logger.factory'),
      $container->get('file_system'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'search_api_synonym_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['search_api_synonym.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config($this->getEditableConfigNames()[0]);

    // Add cron settings.
    $cron = $config->get('cron');

    $options = [];
    foreach ($this->availablePlugins as $id => $source) {
      $definition = $source->getPluginDefinition();
      $options[$id] = $definition['label'];
    }

    $intervals = [0, 900, 1800, 3600, 10800, 21600, 43200, 86400, 604800];
    $intervals = array_combine($intervals, $intervals);
    $intervals = array_map([\Drupal::service('date.formatter'), 'formatInterval'], $intervals);
    $intervals[0] = t('Never');
    $intervals[-1] = t('Every Cron run');

    $form['interval'] = [
      '#type' => 'select',
      '#title' => $this->t('Export synonyms every'),
      '#description' => $this->t('How often should Drupal export synonyms? Use "Every Cron run" to allow export every time Cron is run, this can be helpful for controlling exports using a module such as Ultimate Cron (to export at a set time rather than by interval). Be careful with this, depending on Cron intervals (core, or external) this may run more often than preferred.'),
      '#default_value' => $cron['interval'] ?? 86400,
      '#options' => $intervals,
    ];

    $form['cron'] = [
      '#title' => $this->t('Cron settings'),
      '#type' => 'details',
      '#open' => TRUE,
      '#tree' => TRUE,
      '#states' => [
        'invisible' => [
          ':input[name="interval"]' => ['value' => 0],
        ],
      ],
    ];

    $form['cron']['plugin'] = [
      '#type' => 'select',
      '#title' => $this->t('Synonym export plugin'),
      '#description' => $this->t('Select the export plugin being used by cron.'),
      '#default_value' => $cron['plugin'] ? $cron['plugin'] : '',
      '#options' => $options,
    ];

    $form['cron']['type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Type'),
      '#description' => $this->t('Which synonym type should be exported by cron?'),
      '#default_value' => $cron['type'] ? $cron['type'] : 'all',
      '#options' => [
        'all' => $this->t('All'),
        'synonym' => $this->t('Synonyms'),
        'spelling_error' => $this->t('Spelling errors'),
      ],
    ];

    $form['cron']['filter'] = [
      '#type' => 'radios',
      '#title' => $this->t('Filter'),
      '#description' => $this->t('Which filters should be used when selecting synonyms.'),
      '#default_value' => $cron['filter'] ? $cron['filter'] : 'none',
      '#options' => [
        'none' => $this->t('No filter'),
        'nospace' => $this->t('Synonyms without spaces in the word'),
        'onlyspace' => $this->t('Synonyms with spaces in the word'),
      ],
    ];

    $form['cron']['separate_files'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Separate files'),
      '#description' => $this->t('Export synonyms with and without spaces into separate files.'),
      '#default_value' => $cron['separate_files'] ? $cron['separate_files'] : '',
      '#states' => [
        'visible' => [
          ':radio[name="cron[filter]"]' => ['value' => 'none'],
        ],
      ],
    ];

    $form['cron']['export_if_changed'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Only export if changes'),
      '#description' => $this->t('Only export synonyms if there are either new or changed synonyms since the last export.'),
      '#default_value' => $cron['export_if_changed'] ? $cron['export_if_changed'] : FALSE,
    ];

    $form['cron']['file_export_location'] = [
      '#type' => 'textfield',
      '#title' => $this->t('File Export location'),
      '#default_value' => $cron['file_export_location'] ? $cron['file_export_location'] : '',
      '#description' => $this->t('Configure the synonym file export location. Include scheme, example "public://custom-synonym-folder"'),
      '#required' => FALSE,
    ];

    $form['#validate'][] = [$this, 'validateForm'];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValue('cron');
    $values['interval'] = $form_state->getValue('interval');

    $this->config($this->getEditableConfigNames()[0])
      ->set('cron', $values)
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $folder = $form_state->getValue(['cron', 'file_export_location']);

    if (!empty($folder)) {
      try {
        $result = $this->fileSystem->prepareDirectory(
          $folder,
          FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS
        );
      }
      catch (\Exception $e) {
        $this->loggerFactory->get('search_api_synonym')->error('Directory preparation failed for %dir: @message', [
          '%dir' => $folder,
          '@message' => $e->getMessage(),
        ]);
      }

      if (!$result) {
        $form_state->setError($form['cron']['file_export_location'], $this->t('The configured folder could not be created.'));
      }
    }
  }

}
